<?php

namespace Myriad\Squire;

use GuzzleHttp\Client as Guzzle;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Psr7\Stream;
use Illuminate\Contracts\Filesystem\FileNotFoundException;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Str;
use Myriad\Squire\Contracts\Squire;
use Psr\Http\Message\ResponseInterface;
use SplFileInfo;

class SquireClient implements Squire
{

    /**
     * @var SquireConfiguration
     */
    private $config;

    /**
     * @var Guzzle
     */
    private $guzzle;

    /**
     * @inheritdoc
     */
    public function __construct(SquireConfiguration $config, Guzzle $guzzle)
    {
        $this->config = $config;
        $this->guzzle = $guzzle;
    }

    /**
     * @inheritdoc
     * @throws GuzzleException
     * @throws FileNotFoundException
     */
    public function get(string $path): FileResponse
    {
        $response = $this->doFileRequest('GET', ['query' => ['path' => $path]]);
        return new FileResponse(json_decode($response->getBody(), true));
    }

    /**
     * @inheritdoc
     * @throws GuzzleException
     * @throws FileNotFoundException
     */
    public function getAsStream(string $path)
    {
        $response = $this->doFileRequest('GET', ['query' => ['path' => $path, 'alt' => 'media']]);
        return $response->getBody()->detach();
    }

    /**
     * @param  string  $path
     * @return Stream
     * @throws FileNotFoundException
     * @throws GuzzleException
     */
    public function getAsGuzzleStream(string $path): Stream
    {
        return new Stream($this->getAsStream($path));
    }

    /**
     * @inheritdoc
     * @throws GuzzleException
     * @throws FileNotFoundException
     */
    public function getAsString(string $path): string
    {
        $response = $this->doFileRequest('GET', ['query' => ['path' => $path, 'alt' => 'media']]);
        return $response->getBody()->getContents();
    }

    /**
     * @inheritdoc
     */
    public function putUploadedFile(string $path, UploadedFile $file): FileResponse
    {
        return $this->putStream(
            $path,
            fopen($file->getRealPath(), 'r'),
            $file->getSize(),
            $file->getClientMimeType()
        );
    }

    /**
     * @inheritdoc
     */
    public function putFile(string $path, SplFileInfo $file, string $contentType = 'binary/octet-stream'): FileResponse
    {
        return $this->putStream($path, fopen($file->getRealPath(), 'r'), $file->getSize(), $contentType);
    }

    /**
     * @inheritdoc
     * @throws GuzzleException
     * @throws FileNotFoundException
     */
    public function putStream(
        string $path,
        $stream,
        int $contentLength,
        string $contentType = 'binary/octet-stream'
    ): FileResponse {
        $response = $this->doFileRequest('POST', [
            'query' => ['path' => $path],
            'body' => $stream,
            'headers' => [
                'Content-Type' => $contentType,
                'Content-Length' => $contentLength,
            ],
        ]);
        return new FileResponse(json_decode($response->getBody(), true));
    }

    /**
     * @inheritdoc
     * @throws GuzzleException
     * @throws FileNotFoundException
     */
    public function delete(string $path): bool
    {
        $this->doFileRequest('DELETE', ['query' => ['path' => $path]]);
        return true;
    }

    /**
     * @inheritdoc
     */
    public function image($path): SquireImage
    {
        return new SquireImage($path, $this->getConfiguration());
    }

    /**
     * @inheritdoc
     */
    public function file($path): SquireFile
    {
        return new SquireFile($path, $this->getConfiguration());
    }

    /**
     * @inheritdoc
     */
    public function getConfiguration(): SquireConfiguration
    {
        return $this->config;
    }

    /**
     * @inheritdoc
     */
    public function generateUniquePathForFile(SplFileInfo $file, $prefix = '')
    {
        $extension = $file->getExtension();
        if ($file instanceof UploadedFile) {
            $extension = $file->getClientOriginalExtension();
        }
        return $this->generateUniquePathForExtension($extension, $prefix);
    }

    /**
     * @inheritdoc
     */
    public function generateUniquePathForExtension($extension, $prefix = '')
    {
        if (!empty($prefix) && !Str::endsWith($prefix, '/')) {
            $prefix .= '-';
        }
        return trim($prefix.Str::random(64).'.'.ltrim($extension, '.'), '/.');
    }

    /**
     * @param $method
     * @param  array  $options
     * @return mixed|ResponseInterface
     * @throws FileNotFoundException
     * @throws GuzzleException
     */
    private function doFileRequest($method, $options = [])
    {
        data_set($options, 'connect_timeout', 10);
        data_set($options, 'headers.Authorization', $this->getConfiguration()->getServerToken());
        $url = $this->config->getServerUrl().'/api/v2/files';

        try {
            return $this->guzzle->request($method, $url, $options);
        } catch (ClientException $e) {
            if ($e->getResponse()->getStatusCode() == 404) {
                throw new FileNotFoundException();
            }
            throw $e;
        }
    }
}
