<?php

use GuzzleHttp\Client;
use Illuminate\Contracts\Cache\Repository;
use Illuminate\Foundation\Application;
use Mockery as m;
use Mockery\MockInterface;
use Myriad\Squire\SquireClient;
use Myriad\Squire\SquireConfiguration;
use Orchestra\Testbench\TestCase;
use Psr\Http\Message\RequestInterface;

class SquireTestCase extends TestCase
{

    protected $confUrl = 'https://localhost';

    protected $confToken = 'abc123';

    protected $confDomain = 'https://testcdn.example.com';

    protected $confPrefix = 'test';

    /**
     * Setup the test environment.
     */
    protected function setUp(): void
    {
        global $squireConfig;
        $squireConfig = $this->getMockFullConfiguration();

        // define the config_path function
        if (!function_exists('config_path')) {
            function config_path($path)
            {
                return $path;
            }
        }

        // define the config function and return the default config
        if (!function_exists('config')) {
            function config()
            {
                global $squireConfig;

                return $squireConfig;
            }
        }
    }

    /**
     * Create a mock of the laravel cache
     *
     * @return MockInterface
     */
    protected function getMockCacheRepository()
    {
        return m::mock(Repository::class);
    }

    /**
     * Create a mock of the guzzle client
     *
     * @return MockInterface
     */
    protected function getMockGuzzleClient()
    {
        return m::mock(Client::class);
    }

    /**
     * Create a mock of the guzzle request interface
     *
     * @return MockInterface
     */
    protected function getMockGuzzleRequestInterface()
    {
        return m::mock(RequestInterface::class);
    }

    /**
     * Create the base configuration expected by the client
     *
     * @return array
     */
    protected function getBaseConfiguration(): array
    {
        return [
            'server_url' => $this->confUrl,
            'server_token' => $this->confToken,
        ];
    }

    /**
     * Create the mock provisioning data array fetched from the server
     *
     * @return array
     */
    protected function getMockServerConfiguration(): array
    {
        return [
            'domain' => $this->confDomain,
            'prefix' => $this->confPrefix,
        ];
    }

    /**
     * Create the full configuration data mock
     *
     * @return array
     */
    protected function getMockFullConfiguration(): array
    {
        return array_merge($this->getBaseConfiguration(), $this->getMockServerConfiguration());
    }

    /**
     * Create a filled configuration object
     *
     * @param  null|array  $config
     * @param  null|Repository  $repository
     * @param  null|GuzzleHttp\Client  $guzzle
     *
     * @return SquireConfiguration
     */
    protected function getConfiguration($config = null, $repository = null, $guzzle = null): SquireConfiguration
    {
        return new SquireConfiguration(
            $config ?: $this->getMockFullConfiguration(),
            $repository ?: $this->getMockCacheRepository(),
            $guzzle ?: $this->getMockGuzzleClient()
        );
    }

    /**
     * Create a new SquireClient object
     *
     * @param  null|SquireConfiguration  $config
     * @param  null|GuzzleHttp\Client  $guzzle
     *
     * @return SquireClient
     */
    protected function getSquire($config = null, $guzzle = null)
    {
        return new SquireClient(
            $config ?: $this->getConfiguration(),
            $guzzle ?: $this->getMockGuzzleClient()
        );
    }

    /**
     * Create a Mocked Laravel App Instance
     *
     * @return MockInterface
     */
    protected function getMockedApp()
    {
        return m::mock(new ArrayObject());
    }

}
